@@ -21,7 +21,6 @@ var num;
 var type;
 var chunk;
 var chunks;
-var encoding;
 
 function main(conf) {
   dur = +conf.dur;
@@ -30,7 +29,7 @@ function main(conf) {
   type = conf.type;
   chunks = +conf.chunks;
 
-  chunk = []
+  chunk = [];
   for (var i = 0; i < chunks; i++) {
     chunk.push(new Buffer(Math.round(len / chunks)));
   }
