@@ -1,43 +1,20 @@
-// Copyright Joyent, Inc. and other Node contributors.
-//
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the
-// "Software"), to deal in the Software without restriction, including
-// without limitation the rights to use, copy, modify, merge, publish,
-// distribute, sublicense, and/or sell copies of the Software, and to permit
-// persons to whom the Software is furnished to do so, subject to the
-// following conditions:
-//
-// The above copyright notice and this permission notice shall be included
-// in all copies or substantial portions of the Software.
-//
-// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
-// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
-// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
-// USE OR OTHER DEALINGS IN THE SOFTWARE.
+'use strict';
 
-var util = require('util');
-var net = require('net');
-var EventEmitter = require('events').EventEmitter;
-var HTTPParser = process.binding('http_parser').HTTPParser;
-var assert = require('assert').ok;
+const util = require('util');
+const net = require('net');
+const HTTPParser = process.binding('http_parser').HTTPParser;
+const assert = require('assert').ok;
+const common = require('_http_common');
+const parsers = common.parsers;
+const freeParser = common.freeParser;
+const debug = common.debug;
+const CRLF = common.CRLF;
+const continueExpression = common.continueExpression;
+const chunkExpression = common.chunkExpression;
+const httpSocketSetup = common.httpSocketSetup;
+const OutgoingMessage = require('_http_outgoing').OutgoingMessage;
 
-var common = require('_http_common');
-var parsers = common.parsers;
-var freeParser = common.freeParser;
-var debug = common.debug;
-var CRLF = common.CRLF;
-var continueExpression = common.continueExpression;
-var chunkExpression = common.chunkExpression;
-var httpSocketSetup = common.httpSocketSetup;
-
-var OutgoingMessage = require('_http_outgoing').OutgoingMessage;
-
-
-var STATUS_CODES = exports.STATUS_CODES = {
+const STATUS_CODES = exports.STATUS_CODES = {
   100 : 'Continue',
   101 : 'Switching Protocols',
   102 : 'Processing',                 // RFC 2518, obsoleted by RFC 4918
@@ -49,9 +26,11 @@ var STATUS_CODES = exports.STATUS_CODES = {
   205 : 'Reset Content',
   206 : 'Partial Content',
   207 : 'Multi-Status',               // RFC 4918
+  208 : 'Already Reported',
+  226 : 'IM Used',
   300 : 'Multiple Choices',
   301 : 'Moved Permanently',
-  302 : 'Moved Temporarily',
+  302 : 'Found',
   303 : 'See Other',
   304 : 'Not Modified',
   305 : 'Use Proxy',
@@ -65,17 +44,18 @@ var STATUS_CODES = exports.STATUS_CODES = {
   405 : 'Method Not Allowed',
   406 : 'Not Acceptable',
   407 : 'Proxy Authentication Required',
-  408 : 'Request Time-out',
+  408 : 'Request Timeout',
   409 : 'Conflict',
   410 : 'Gone',
   411 : 'Length Required',
   412 : 'Precondition Failed',
-  413 : 'Request Entity Too Large',
-  414 : 'Request-URI Too Large',
+  413 : 'Payload Too Large',
+  414 : 'URI Too Long',
   415 : 'Unsupported Media Type',
-  416 : 'Requested Range Not Satisfiable',
+  416 : 'Range Not Satisfiable',
   417 : 'Expectation Failed',
   418 : 'I\'m a teapot',              // RFC 2324
+  421 : 'Misdirected Request',
   422 : 'Unprocessable Entity',       // RFC 4918
   423 : 'Locked',                     // RFC 4918
   424 : 'Failed Dependency',          // RFC 4918
@@ -83,20 +63,24 @@ var STATUS_CODES = exports.STATUS_CODES = {
   426 : 'Upgrade Required',           // RFC 2817
   428 : 'Precondition Required',      // RFC 6585
   429 : 'Too Many Requests',          // RFC 6585
-  431 : 'Request Header Fields Too Large',// RFC 6585
+  431 : 'Request Header Fields Too Large', // RFC 6585
+  451 : 'Unavailable For Legal Reasons',
   500 : 'Internal Server Error',
   501 : 'Not Implemented',
   502 : 'Bad Gateway',
   503 : 'Service Unavailable',
-  504 : 'Gateway Time-out',
+  504 : 'Gateway Timeout',
   505 : 'HTTP Version Not Supported',
   506 : 'Variant Also Negotiates',    // RFC 2295
   507 : 'Insufficient Storage',       // RFC 4918
+  508 : 'Loop Detected',
   509 : 'Bandwidth Limit Exceeded',
   510 : 'Not Extended',               // RFC 2774
   511 : 'Network Authentication Required' // RFC 6585
 };
 
+const kOnExecute = HTTPParser.kOnExecute | 0;
+
 
 function ServerResponse(req) {
   OutgoingMessage.call(this);
@@ -114,12 +98,12 @@ util.inherits(ServerResponse, OutgoingMessage);
 
 ServerResponse.prototype._finish = function() {
   DTRACE_HTTP_SERVER_RESPONSE(this.connection);
+  LTTNG_HTTP_SERVER_RESPONSE(this.connection);
   COUNTER_HTTP_SERVER_RESPONSE();
   OutgoingMessage.prototype._finish.call(this);
 };
 
 
-
 exports.ServerResponse = ServerResponse;
 
 ServerResponse.prototype.statusCode = 200;
@@ -133,7 +117,8 @@ function onServerResponseClose() {
   // array. That is, in the example below, b still gets called even though
   // it's been removed by a:
   //
-  //   var obj = new events.EventEmitter;
+  //   var EventEmitter = require('events');
+  //   var obj = new EventEmitter();
   //   obj.on('event', a);
   //   obj.on('event', b);
   //   function a() { obj.removeListener('event', b) }
@@ -175,7 +160,7 @@ ServerResponse.prototype._implicitHeader = function() {
 ServerResponse.prototype.writeHead = function(statusCode, reason, obj) {
   var headers;
 
-  if (util.isString(reason)) {
+  if (typeof reason === 'string') {
     // writeHead(statusCode, reasonPhrase[, headers])
     this.statusMessage = reason;
   } else {
@@ -242,9 +227,11 @@ function Server(requestListener) {
     this.addListener('request', requestListener);
   }
 
+  /* eslint-disable max-len */
   // Similar option to this. Too lazy to write my own docs.
   // http://www.squid-cache.org/Doc/config/half_closed_clients/
   // http://wiki.squid-cache.org/SquidFaq/InnerWorkings#What_is_a_half-closed_filedescriptor.3F
+  /* eslint-enable max-len */
   this.httpAllowHalfOpen = false;
 
   this.addListener('connection', connectionListener);
@@ -254,6 +241,8 @@ function Server(requestListener) {
   });
 
   this.timeout = 2 * 60 * 1000;
+
+  this._pendingResponseData = 0;
 }
 util.inherits(Server, net.Server);
 
@@ -262,6 +251,7 @@ Server.prototype.setTimeout = function(msecs, callback) {
   this.timeout = msecs;
   if (callback)
     this.on('timeout', callback);
+  return this;
 };
 
 
@@ -272,6 +262,17 @@ function connectionListener(socket) {
   var self = this;
   var outgoing = [];
   var incoming = [];
+  var outgoingData = 0;
+
+  function updateOutgoingData(delta) {
+    // `outgoingData` is an approximate amount of bytes queued through all
+    // inactive responses. If more data than the high watermark is queued - we
+    // need to pause TCP socket/HTTP parser, and wait until the data will be
+    // sent to the client.
+    outgoingData += delta;
+    if (socket._paused && outgoingData < socket._writableState.highWaterMark)
+      return socketOnDrain();
+  }
 
   function abortIncoming() {
     while (incoming.length) {
@@ -319,7 +320,7 @@ function connectionListener(socket) {
   parser.incoming = null;
 
   // Propagate headers limit from server instance to parser
-  if (util.isNumber(this.maxHeadersCount)) {
+  if (typeof this.maxHeadersCount === 'number') {
     parser.maxHeaderPairs = this.maxHeadersCount << 1;
   } else {
     // Set default value because parser may be reused from FreeList
@@ -332,6 +333,23 @@ function connectionListener(socket) {
   socket.on('end', socketOnEnd);
   socket.on('data', socketOnData);
 
+  // We are consuming socket, so it won't get any actual data
+  socket.on('resume', onSocketResume);
+  socket.on('pause', onSocketPause);
+
+  socket.on('drain', socketOnDrain);
+
+  // Override on to unconsume on `data`, `readable` listeners
+  socket.on = socketOnWrap;
+
+  var external = socket._handle._externalStream;
+  if (external) {
+    parser._consumed = true;
+    parser.consume(external);
+  }
+  external = null;
+  parser[kOnExecute] = onParserExecute;
+
   // TODO(isaacs): Move all these functions out of here
   function socketOnError(e) {
     self.emit('clientError', e, this);
@@ -341,6 +359,16 @@ function connectionListener(socket) {
     assert(!socket._paused);
     debug('SERVER socketOnData %d', d.length);
     var ret = parser.execute(d);
+
+    onParserExecuteCommon(ret, d);
+  }
+
+  function onParserExecute(ret, d) {
+    debug('SERVER socketOnParserExecute %d', ret);
+    onParserExecuteCommon(ret, undefined);
+  }
+
+  function onParserExecuteCommon(ret, d) {
     if (ret instanceof Error) {
       debug('parse error');
       socket.destroy(ret);
@@ -350,15 +378,19 @@ function connectionListener(socket) {
       var req = parser.incoming;
       debug('SERVER upgrade or connect', req.method);
 
+      if (!d)
+        d = parser.getCurrentBuffer();
+
       socket.removeListener('data', socketOnData);
       socket.removeListener('end', socketOnEnd);
       socket.removeListener('close', serverSocketCloseListener);
+      unconsume(parser, socket);
       parser.finish();
       freeParser(parser, req, null);
       parser = null;
 
       var eventName = req.method === 'CONNECT' ? 'connect' : 'upgrade';
-      if (EventEmitter.listenerCount(self, eventName) > 0) {
+      if (self.listenerCount(eventName) > 0) {
         debug('SERVER have listener for %s', eventName);
         var bodyHead = d.slice(bytesParsed, d.length);
 
@@ -408,15 +440,16 @@ function connectionListener(socket) {
 
   socket._paused = false;
   function socketOnDrain() {
+    var needPause = outgoingData > socket._writableState.highWaterMark;
+
     // If we previously paused, then start reading again.
-    if (socket._paused) {
+    if (socket._paused && !needPause) {
       socket._paused = false;
       if (socket.parser)
         socket.parser.resume();
       socket.resume();
     }
   }
-  socket.on('drain', socketOnDrain);
 
   function parserOnIncoming(req, shouldKeepAlive) {
     incoming.push(req);
@@ -425,7 +458,8 @@ function connectionListener(socket) {
     // so that we don't become overwhelmed by a flood of
     // pipelined requests that may never be resolved.
     if (!socket._paused) {
-      var needPause = socket._writableState.needDrain;
+      var needPause = socket._writableState.needDrain ||
+          outgoingData >= socket._writableState.highWaterMark;
       if (needPause) {
         socket._paused = true;
         // We also need to pause the parser, but don't do that until after
@@ -436,9 +470,11 @@ function connectionListener(socket) {
     }
 
     var res = new ServerResponse(req);
+    res._onPendingData = updateOutgoingData;
 
     res.shouldKeepAlive = shouldKeepAlive;
     DTRACE_HTTP_SERVER_REQUEST(req, socket);
+    LTTNG_HTTP_SERVER_REQUEST(req, socket);
     COUNTER_HTTP_SERVER_REQUEST();
 
     if (socket._httpMessage) {
@@ -449,8 +485,8 @@ function connectionListener(socket) {
     }
 
     // When we're finished writing the response, check if this is the last
-    // respose, if so destroy the socket.
-    res.on('prefinish', resOnFinish);
+    // response, if so destroy the socket.
+    res.on('finish', resOnFinish);
     function resOnFinish() {
       // Usually the first incoming element should be our request.  it may
       // be that in the case abortIncoming() was called that the incoming
@@ -478,15 +514,24 @@ function connectionListener(socket) {
       }
     }
 
-    if (!util.isUndefined(req.headers.expect) &&
-        (req.httpVersionMajor == 1 && req.httpVersionMinor == 1) &&
-        continueExpression.test(req.headers['expect'])) {
-      res._expect_continue = true;
-      if (EventEmitter.listenerCount(self, 'checkContinue') > 0) {
-        self.emit('checkContinue', req, res);
+    if (req.headers.expect !== undefined &&
+        (req.httpVersionMajor == 1 && req.httpVersionMinor == 1)) {
+      if (continueExpression.test(req.headers.expect)) {
+        res._expect_continue = true;
+
+        if (self.listenerCount('checkContinue') > 0) {
+          self.emit('checkContinue', req, res);
+        } else {
+          res.writeContinue();
+          self.emit('request', req, res);
+        }
       } else {
-        res.writeContinue();
-        self.emit('request', req, res);
+        if (self.listenerCount('checkExpectation') > 0) {
+          self.emit('checkExpectation', req, res);
+        } else {
+          res.writeHead(417);
+          res.end();
+        }
       }
     } else {
       self.emit('request', req, res);
@@ -495,3 +540,51 @@ function connectionListener(socket) {
   }
 }
 exports._connectionListener = connectionListener;
+
+function onSocketResume() {
+  // It may seem that the socket is resumed, but this is an enemy's trick to
+  // deceive us! `resume` is emitted asynchronously, and may be called from
+  // `incoming.readStart()`. Stop the socket again here, just to preserve the
+  // state.
+  //
+  // We don't care about stream semantics for the consumed socket anyway.
+  if (this._paused) {
+    this.pause();
+    return;
+  }
+
+  if (this._handle && !this._handle.reading) {
+    this._handle.reading = true;
+    this._handle.readStart();
+  }
+}
+
+function onSocketPause() {
+  if (this._handle && this._handle.reading) {
+    this._handle.reading = false;
+    this._handle.readStop();
+  }
+}
+
+function unconsume(parser, socket) {
+  if (socket._handle) {
+    if (parser._consumed)
+      parser.unconsume(socket._handle._externalStream);
+    parser._consumed = false;
+    socket.removeListener('pause', onSocketPause);
+    socket.removeListener('resume', onSocketResume);
+  }
+}
+
+function socketOnWrap(ev, fn) {
+  var res = net.Socket.prototype.on.call(this, ev, fn);
+  if (!this.parser) {
+    this.on = net.Socket.prototype.on;
+    return res;
+  }
+
+  if (ev === 'data' || ev === 'readable')
+    unconsume(this.parser, this);
+
+  return res;
+}
