@@ -1,217 +1,324 @@
-// Copyright Joyent, Inc. and other Node contributors.
-//
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the
-// "Software"), to deal in the Software without restriction, including
-// without limitation the rights to use, copy, modify, merge, publish,
-// distribute, sublicense, and/or sell copies of the Software, and to permit
-// persons to whom the Software is furnished to do so, subject to the
-// following conditions:
-//
-// The above copyright notice and this permission notice shall be included
-// in all copies or substantial portions of the Software.
-//
-// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
-// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
-// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
-// USE OR OTHER DEALINGS IN THE SOFTWARE.
+/* eslint-disable require-buffer */
+'use strict';
 
-var buffer = process.binding('buffer');
-var smalloc = process.binding('smalloc');
-var util = require('util');
-var alloc = smalloc.alloc;
-var truncate = smalloc.truncate;
-var sliceOnto = smalloc.sliceOnto;
-var kMaxLength = smalloc.kMaxLength;
-var internal = {};
+const binding = process.binding('buffer');
+const internalUtil = require('internal/util');
+const bindingObj = {};
 
 exports.Buffer = Buffer;
 exports.SlowBuffer = SlowBuffer;
 exports.INSPECT_MAX_BYTES = 50;
+exports.kMaxLength = binding.kMaxLength;
 
+const kFromErrorMsg = 'must start with number, buffer, array or string';
 
 Buffer.poolSize = 8 * 1024;
 var poolSize, poolOffset, allocPool;
 
 
+binding.setupBufferJS(Buffer.prototype, bindingObj);
+
+const swap16n = binding.swap16;
+const swap32n = binding.swap32;
+
+function swap(b, n, m) {
+  const i = b[n];
+  b[n] = b[m];
+  b[m] = i;
+}
+
+Buffer.prototype.swap16 = function swap16() {
+  // For Buffer.length < 512, it's generally faster to
+  // do the swap in javascript. For larger buffers,
+  // dropping down to the native code is faster.
+  const len = this.length;
+  if (len % 2 !== 0)
+    throw new RangeError('Buffer size must be a multiple of 16-bits');
+  if (len < 512) {
+    for (var i = 0; i < len; i += 2)
+      swap(this, i, i + 1);
+    return this;
+  }
+  return swap16n.apply(this);
+};
+
+Buffer.prototype.swap32 = function swap32() {
+  // For Buffer.length < 1024, it's generally faster to
+  // do the swap in javascript. For larger buffers,
+  // dropping down to the native code is faster.
+  const len = this.length;
+  if (len % 4 !== 0)
+    throw new RangeError('Buffer size must be a multiple of 32-bits');
+  if (len < 1024) {
+    for (var i = 0; i < len; i += 4) {
+      swap(this, i, i + 3);
+      swap(this, i + 1, i + 2);
+    }
+    return this;
+  }
+  return swap32n.apply(this);
+};
+
+const flags = bindingObj.flags;
+const kNoZeroFill = 0;
+
+function createBuffer(size) {
+  const ui8 = new Uint8Array(size);
+  Object.setPrototypeOf(ui8, Buffer.prototype);
+  return ui8;
+}
+
 function createPool() {
   poolSize = Buffer.poolSize;
-  allocPool = alloc({}, poolSize);
+  if (poolSize > 0)
+    flags[kNoZeroFill] = 1;
+  allocPool = createBuffer(poolSize);
   poolOffset = 0;
 }
 createPool();
 
 
-function Buffer(subject, encoding) {
-  if (!util.isBuffer(this))
-    return new Buffer(subject, encoding);
-
-  if (util.isNumber(subject)) {
-    this.length = +subject;
-
-  } else if (util.isString(subject)) {
-    if (!util.isString(encoding) || encoding.length === 0)
-      encoding = 'utf8';
-    this.length = Buffer.byteLength(subject, encoding);
-
-  // Handle Arrays, Buffers, Uint8Arrays or JSON.
-  } else if (util.isObject(subject)) {
-    if (subject.type === 'Buffer' && util.isArray(subject.data))
-      subject = subject.data;
-    this.length = +subject.length;
-
-  } else {
-    throw new TypeError('must start with number, buffer, array or string');
-  }
-
-  if (this.length > kMaxLength) {
-    throw new RangeError('Attempt to allocate Buffer larger than maximum ' +
-                         'size: 0x' + kMaxLength.toString(16) + ' bytes');
-  }
-
-  if (this.length < 0)
-    this.length = 0;
-  else
-    this.length >>>= 0;  // Coerce to uint32.
-
-  this.parent = undefined;
-  if (this.length <= (Buffer.poolSize >>> 1) && this.length > 0) {
-    if (this.length > poolSize - poolOffset)
-      createPool();
-    this.parent = sliceOnto(allocPool,
-                            this,
-                            poolOffset,
-                            poolOffset + this.length);
-    poolOffset += this.length;
-
-    // Ensure aligned slices
-    if (poolOffset & 0x7) {
-      poolOffset |= 0x7;
-      poolOffset++;
-    }
-  } else {
-    alloc(this, this.length);
-  }
-
-  if (util.isNumber(subject)) {
-    return;
-  }
-
-  if (util.isString(subject)) {
-    // In the case of base64 it's possible that the size of the buffer
-    // allocated was slightly too large. In this case we need to rewrite
-    // the length to the actual length written.
-    var len = this.write(subject, encoding);
-    // Buffer was truncated after decode, realloc internal ExternalArray
-    if (len !== this.length) {
-      var prevLen = this.length;
-      this.length = len;
-      truncate(this, this.length);
-      // Only need to readjust the poolOffset if the allocation is a slice.
-      if (this.parent != undefined)
-        poolOffset -= (prevLen - len);
-    }
-
-  } else if (util.isBuffer(subject)) {
-    subject.copy(this, 0, 0, this.length);
-
-  } else if (util.isNumber(subject.length) || util.isArray(subject)) {
-    // Really crappy way to handle Uint8Arrays, but V8 doesn't give a simple
-    // way to access the data from the C++ API.
-    for (var i = 0; i < this.length; i++)
-      this[i] = subject[i];
+function alignPool() {
+  // Ensure aligned slices
+  if (poolOffset & 0x7) {
+    poolOffset |= 0x7;
+    poolOffset++;
   }
 }
 
 
-function SlowBuffer(length) {
-  length = length >>> 0;
-  if (length > kMaxLength) {
-    throw new RangeError('Attempt to allocate Buffer larger than maximum ' +
-                         'size: 0x' + kMaxLength.toString(16) + ' bytes');
+function Buffer(arg, encodingOrOffset, length) {
+  // Common case.
+  if (typeof arg === 'number')
+    return Buffer.allocUnsafe(arg);
+  return Buffer.from(arg, encodingOrOffset, length);
+}
+
+/**
+ * Functionally equivalent to Buffer(arg, encoding) but throws a TypeError
+ * if value is a number.
+ * Buffer.from(str[, encoding])
+ * Buffer.from(array)
+ * Buffer.from(buffer)
+ * Buffer.from(arrayBuffer[, byteOffset[, length]])
+ **/
+Buffer.from = function(value, encodingOrOffset, length) {
+  if (typeof value === 'number')
+    throw new TypeError('"value" argument must not be a number');
+
+  if (value instanceof ArrayBuffer)
+    return fromArrayBuffer(value, encodingOrOffset, length);
+
+  if (typeof value === 'string')
+    return fromString(value, encodingOrOffset);
+
+  return fromObject(value);
+};
+
+Object.setPrototypeOf(Buffer.prototype, Uint8Array.prototype);
+Object.setPrototypeOf(Buffer, Uint8Array);
+
+/**
+ * Creates a new filled Buffer instance.
+ * alloc(size[, fill[, encoding]])
+ **/
+Buffer.alloc = function(size, fill, encoding) {
+  if (typeof size !== 'number')
+    throw new TypeError('"size" argument must be a number');
+  if (size <= 0)
+    return createBuffer(size);
+  if (fill !== undefined) {
+    // Since we are filling anyway, don't zero fill initially.
+    flags[kNoZeroFill] = 1;
+    // Only pay attention to encoding if it's a string. This
+    // prevents accidentally sending in a number that would
+    // be interpretted as a start offset.
+    return typeof encoding === 'string' ?
+        createBuffer(size).fill(fill, encoding) :
+        createBuffer(size).fill(fill);
   }
-  var b = new NativeBuffer(length);
-  alloc(b, length);
+  flags[kNoZeroFill] = 0;
+  return createBuffer(size);
+};
+
+/**
+ * Equivalent to Buffer(num), by default creates a non-zero-filled Buffer
+ * instance. If `--zero-fill-buffers` is set, will zero-fill the buffer.
+ **/
+Buffer.allocUnsafe = function(size) {
+  if (typeof size !== 'number')
+    throw new TypeError('"size" argument must be a number');
+  if (size > 0)
+    flags[kNoZeroFill] = 1;
+  return allocate(size);
+};
+
+function SlowBuffer(length) {
+  if (+length != length)
+    length = 0;
+  if (length > 0)
+    flags[kNoZeroFill] = 1;
+  return createBuffer(+length);
+}
+
+Object.setPrototypeOf(SlowBuffer.prototype, Uint8Array.prototype);
+Object.setPrototypeOf(SlowBuffer, Uint8Array);
+
+
+function allocate(size) {
+  if (size === 0) {
+    return createBuffer(size);
+  }
+  if (size < (Buffer.poolSize >>> 1)) {
+    if (size > (poolSize - poolOffset))
+      createPool();
+    var b = allocPool.slice(poolOffset, poolOffset + size);
+    poolOffset += size;
+    alignPool();
+    return b;
+  } else {
+    // Even though this is checked above, the conditional is a safety net and
+    // sanity check to prevent any subsequent typed array allocation from not
+    // being zero filled.
+    if (size > 0)
+      flags[kNoZeroFill] = 1;
+    return createBuffer(size);
+  }
+}
+
+
+function fromString(string, encoding) {
+  if (typeof encoding !== 'string' || encoding === '')
+    encoding = 'utf8';
+
+  var length = byteLength(string, encoding);
+  if (length >= (Buffer.poolSize >>> 1))
+    return binding.createFromString(string, encoding);
+
+  if (length > (poolSize - poolOffset))
+    createPool();
+  var actual = allocPool.write(string, poolOffset, encoding);
+  var b = allocPool.slice(poolOffset, poolOffset + actual);
+  poolOffset += actual;
+  alignPool();
   return b;
 }
 
-
-// Bypass all checks for instantiating unallocated Buffer required for
-// Objects created in C++. Significantly faster than calling the Buffer
-// function.
-function NativeBuffer(length) {
-  this.length = length >>> 0;
-  // Set this to keep the object map the same.
-  this.parent = undefined;
+function fromArrayLike(obj) {
+  const length = obj.length;
+  const b = allocate(length);
+  for (var i = 0; i < length; i++)
+    b[i] = obj[i] & 255;
+  return b;
 }
-NativeBuffer.prototype = Buffer.prototype;
 
+function fromArrayBuffer(obj, byteOffset, length) {
+  byteOffset >>>= 0;
 
-// add methods to Buffer prototype
-buffer.setupBufferJS(NativeBuffer, internal);
+  if (typeof length === 'undefined')
+    return binding.createFromArrayBuffer(obj, byteOffset);
+
+  length >>>= 0;
+  return binding.createFromArrayBuffer(obj, byteOffset, length);
+}
+
+function fromObject(obj) {
+  if (obj instanceof Buffer) {
+    const b = allocate(obj.length);
+
+    if (b.length === 0)
+      return b;
+
+    obj.copy(b, 0, 0, obj.length);
+    return b;
+  }
+
+  if (obj) {
+    if (obj.buffer instanceof ArrayBuffer || 'length' in obj) {
+      if (typeof obj.length !== 'number' || obj.length !== obj.length) {
+        return allocate(0);
+      }
+      return fromArrayLike(obj);
+    }
+
+    if (obj.type === 'Buffer' && Array.isArray(obj.data)) {
+      return fromArrayLike(obj.data);
+    }
+  }
+
+  throw new TypeError(kFromErrorMsg);
+}
 
 
 // Static methods
 
 Buffer.isBuffer = function isBuffer(b) {
-  return util.isBuffer(b);
+  return b instanceof Buffer;
 };
 
 
 Buffer.compare = function compare(a, b) {
   if (!(a instanceof Buffer) ||
-      !(b instanceof Buffer))
+      !(b instanceof Buffer)) {
     throw new TypeError('Arguments must be Buffers');
+  }
 
-  return internal.compare(a, b);
+  if (a === b) {
+    return 0;
+  }
+
+  return binding.compare(a, b);
 };
 
 
 Buffer.isEncoding = function(encoding) {
-  switch ((encoding + '').toLowerCase()) {
-    case 'hex':
-    case 'utf8':
-    case 'utf-8':
-    case 'ascii':
-    case 'binary':
-    case 'base64':
-    case 'ucs2':
-    case 'ucs-2':
-    case 'utf16le':
-    case 'utf-16le':
-    case 'raw':
-      return true;
+  var loweredCase = false;
+  for (;;) {
+    switch (encoding) {
+      case 'hex':
+      case 'utf8':
+      case 'utf-8':
+      case 'ascii':
+      case 'binary':
+      case 'base64':
+      case 'ucs2':
+      case 'ucs-2':
+      case 'utf16le':
+      case 'utf-16le':
+        return true;
 
-    default:
-      return false;
+      default:
+        if (loweredCase)
+          return false;
+        encoding = ('' + encoding).toLowerCase();
+        loweredCase = true;
+    }
   }
 };
 
 
 Buffer.concat = function(list, length) {
-  if (!util.isArray(list))
-    throw new TypeError('list argument must be an Array of Buffers.');
+  var i;
+  if (!Array.isArray(list))
+    throw new TypeError('list argument must be an Array of Buffers');
 
-  if (util.isUndefined(length)) {
+  if (list.length === 0)
+    return Buffer.alloc(0);
+
+  if (length === undefined) {
     length = 0;
-    for (var i = 0; i < list.length; i++)
+    for (i = 0; i < list.length; i++)
       length += list[i].length;
   } else {
     length = length >>> 0;
   }
 
-  if (list.length === 0)
-    return new Buffer(0);
-  else if (list.length === 1)
-    return list[0];
-
-  var buffer = new Buffer(length);
+  var buffer = Buffer.allocUnsafe(length);
   var pos = 0;
-  for (var i = 0; i < list.length; i++) {
+  for (i = 0; i < list.length; i++) {
     var buf = list[i];
+    if (!Buffer.isBuffer(buf))
+      throw new TypeError('list argument must be an Array of Buffers');
     buf.copy(buffer, pos);
     pos += buf.length;
   }
@@ -220,42 +327,123 @@ Buffer.concat = function(list, length) {
 };
 
 
-Buffer.byteLength = function(str, enc) {
-  var ret;
-  str = str + '';
-  switch (enc) {
-    case 'ascii':
-    case 'binary':
-    case 'raw':
-      ret = str.length;
-      break;
-    case 'ucs2':
-    case 'ucs-2':
-    case 'utf16le':
-    case 'utf-16le':
-      ret = str.length * 2;
-      break;
-    case 'hex':
-      ret = str.length >>> 1;
-      break;
-    default:
-      ret = internal.byteLength(str, enc);
+function base64ByteLength(str, bytes) {
+  // Handle padding
+  if (str.charCodeAt(bytes - 1) === 0x3D)
+    bytes--;
+  if (bytes > 1 && str.charCodeAt(bytes - 1) === 0x3D)
+    bytes--;
+
+  // Base64 ratio: 3/4
+  return (bytes * 3) >>> 2;
+}
+
+
+function byteLength(string, encoding) {
+  if (typeof string !== 'string') {
+    if (ArrayBuffer.isView(string) || string instanceof ArrayBuffer)
+      return string.byteLength;
+
+    string = '' + string;
   }
-  return ret;
-};
+
+  var len = string.length;
+  if (len === 0)
+    return 0;
+
+  // Use a for loop to avoid recursion
+  var loweredCase = false;
+  for (;;) {
+    switch (encoding) {
+      case 'ascii':
+      case 'binary':
+        return len;
+
+      case 'utf8':
+      case 'utf-8':
+      case undefined:
+        return binding.byteLengthUtf8(string);
+
+      case 'ucs2':
+      case 'ucs-2':
+      case 'utf16le':
+      case 'utf-16le':
+        return len * 2;
+
+      case 'hex':
+        return len >>> 1;
+
+      case 'base64':
+        return base64ByteLength(string, len);
+
+      default:
+        // The C++ binding defaulted to UTF8, we should too.
+        if (loweredCase)
+          return binding.byteLengthUtf8(string);
+
+        encoding = ('' + encoding).toLowerCase();
+        loweredCase = true;
+    }
+  }
+}
+
+Buffer.byteLength = byteLength;
 
 
-// toString(encoding, start=0, end=buffer.length)
-Buffer.prototype.toString = function(encoding, start, end) {
+// For backwards compatibility.
+Object.defineProperty(Buffer.prototype, 'parent', {
+  enumerable: true,
+  get: function() {
+    if (!(this instanceof Buffer))
+      return undefined;
+    if (this.byteLength === 0 ||
+        this.byteLength === this.buffer.byteLength) {
+      return undefined;
+    }
+    return this.buffer;
+  }
+});
+Object.defineProperty(Buffer.prototype, 'offset', {
+  enumerable: true,
+  get: function() {
+    if (!(this instanceof Buffer))
+      return undefined;
+    return this.byteOffset;
+  }
+});
+
+
+function slowToString(encoding, start, end) {
   var loweredCase = false;
 
-  start = start >>> 0;
-  end = util.isUndefined(end) || end === Infinity ? this.length : end >>> 0;
+  // No need to verify that "this.length <= MAX_UINT32" since it's a read-only
+  // property of a typed array.
+
+  // This behaves neither like String nor Uint8Array in that we set start/end
+  // to their upper/lower bounds if the value passed is out of range.
+  // undefined is handled specially as per ECMA-262 6th Edition,
+  // Section 13.3.3.7 Runtime Semantics: KeyedBindingInitialization.
+  if (start === undefined || start < 0)
+    start = 0;
+  // Return early if start > this.length. Done here to prevent potential uint32
+  // coercion fail below.
+  if (start > this.length)
+    return '';
+
+  if (end === undefined || end > this.length)
+    end = this.length;
+
+  if (end <= 0)
+    return '';
+
+  // Force coersion to uint32. This will also coerce falsey/NaN values to 0.
+  end >>>= 0;
+  start >>>= 0;
+
+  if (end <= start)
+    return '';
 
   if (!encoding) encoding = 'utf8';
-  if (start < 0) start = 0;
-  if (end > this.length) end = this.length;
-  if (end <= start) return '';
 
   while (true) {
     switch (encoding) {
@@ -288,6 +476,19 @@ Buffer.prototype.toString = function(encoding, start, end) {
         loweredCase = true;
     }
   }
+}
+
+
+Buffer.prototype.toString = function() {
+  let result;
+  if (arguments.length === 0) {
+    result = this.utf8Slice(0, this.length);
+  } else {
+    result = slowToString.apply(this, arguments);
+  }
+  if (result === undefined)
+    throw new Error('toString failed');
+  return result;
 };
 
 
@@ -295,7 +496,10 @@ Buffer.prototype.equals = function equals(b) {
   if (!(b instanceof Buffer))
     throw new TypeError('Argument must be a Buffer');
 
-  return internal.compare(this, b) === 0;
+  if (this === b)
+    return true;
+
+  return binding.compare(this, b) === 0;
 };
 
 
@@ -316,65 +520,156 @@ Buffer.prototype.compare = function compare(b) {
   if (!(b instanceof Buffer))
     throw new TypeError('Argument must be a Buffer');
 
-  return internal.compare(this, b);
+  if (this === b)
+    return 0;
+
+  return binding.compare(this, b);
+};
+
+function slowIndexOf(buffer, val, byteOffset, encoding) {
+  var loweredCase = false;
+  for (;;) {
+    switch (encoding) {
+      case 'utf8':
+      case 'utf-8':
+      case 'ucs2':
+      case 'ucs-2':
+      case 'utf16le':
+      case 'utf-16le':
+      case 'binary':
+        return binding.indexOfString(buffer, val, byteOffset, encoding);
+
+      case 'base64':
+      case 'ascii':
+      case 'hex':
+        return binding.indexOfBuffer(
+            buffer, Buffer.from(val, encoding), byteOffset, encoding);
+
+      default:
+        if (loweredCase) {
+          throw new TypeError('Unknown encoding: ' + encoding);
+        }
+
+        encoding = ('' + encoding).toLowerCase();
+        loweredCase = true;
+    }
+  }
+}
+
+Buffer.prototype.indexOf = function indexOf(val, byteOffset, encoding) {
+  if (typeof byteOffset === 'string') {
+    encoding = byteOffset;
+    byteOffset = 0;
+  } else if (byteOffset > 0x7fffffff) {
+    byteOffset = 0x7fffffff;
+  } else if (byteOffset < -0x80000000) {
+    byteOffset = -0x80000000;
+  }
+  byteOffset >>= 0;
+
+  if (typeof val === 'string') {
+    if (encoding === undefined) {
+      return binding.indexOfString(this, val, byteOffset, encoding);
+    }
+    return slowIndexOf(this, val, byteOffset, encoding);
+  } else if (val instanceof Buffer) {
+    return binding.indexOfBuffer(this, val, byteOffset, encoding);
+  } else if (typeof val === 'number') {
+    return binding.indexOfNumber(this, val, byteOffset);
+  }
+
+  throw new TypeError('val must be string, number or Buffer');
 };
 
 
-Buffer.prototype.fill = function fill(val, start, end) {
-  start = start >> 0;
-  end = (end === undefined) ? this.length : end >> 0;
+Buffer.prototype.includes = function includes(val, byteOffset, encoding) {
+  return this.indexOf(val, byteOffset, encoding) !== -1;
+};
 
+
+// Usage:
+//    buffer.fill(number[, offset[, end]])
+//    buffer.fill(buffer[, offset[, end]])
+//    buffer.fill(string[, offset[, end]][, encoding])
+Buffer.prototype.fill = function fill(val, start, end, encoding) {
+  // Handle string cases:
+  if (typeof val === 'string') {
+    if (typeof start === 'string') {
+      encoding = start;
+      start = 0;
+      end = this.length;
+    } else if (typeof end === 'string') {
+      encoding = end;
+      end = this.length;
+    }
+    if (val.length === 1) {
+      var code = val.charCodeAt(0);
+      if (code < 256)
+        val = code;
+    }
+    if (val.length === 0) {
+      // Previously, if val === '', the Buffer would not fill,
+      // which is rather surprising.
+      val = 0;
+    }
+    if (encoding !== undefined && typeof encoding !== 'string') {
+      throw new TypeError('encoding must be a string');
+    }
+    if (typeof encoding === 'string' && !Buffer.isEncoding(encoding)) {
+      throw new TypeError('Unknown encoding: ' + encoding);
+    }
+
+  } else if (typeof val === 'number') {
+    val = val & 255;
+  }
+
+  // Invalid ranges are not set to a default, so can range check early.
   if (start < 0 || end > this.length)
     throw new RangeError('out of range index');
+
   if (end <= start)
     return this;
 
-  if (typeof val !== 'string') {
-    val = val >>> 0;
-  } else if (val.length === 1) {
-    var code = val.charCodeAt(0);
-    if (code < 256)
-      val = code;
-  }
+  start = start >>> 0;
+  end = end === undefined ? this.length : end >>> 0;
 
-  internal.fill(this, val, start, end);
+  binding.fill(this, val, start, end, encoding);
 
   return this;
 };
 
 
 // XXX remove in v0.13
-Buffer.prototype.get = util.deprecate(function get(offset) {
+Buffer.prototype.get = internalUtil.deprecate(function get(offset) {
   offset = ~~offset;
   if (offset < 0 || offset >= this.length)
     throw new RangeError('index out of range');
   return this[offset];
-}, '.get() is deprecated. Access using array indexes instead.');
+}, 'Buffer.get is deprecated. Use array indexes instead.');
 
 
 // XXX remove in v0.13
-Buffer.prototype.set = util.deprecate(function set(offset, v) {
+Buffer.prototype.set = internalUtil.deprecate(function set(offset, v) {
   offset = ~~offset;
   if (offset < 0 || offset >= this.length)
     throw new RangeError('index out of range');
   return this[offset] = v;
-}, '.set() is deprecated. Set using array indexes instead.');
+}, 'Buffer.set is deprecated. Use array indexes instead.');
 
 
-// TODO(trevnorris): fix these checks to follow new standard
-// write(string, offset = 0, length = buffer.length, encoding = 'utf8')
 var writeWarned = false;
-var writeMsg = '.write(string, encoding, offset, length) is deprecated.' +
-               ' Use write(string[, offset[, length]][, encoding]) instead.';
+const writeMsg = 'Buffer.write(string, encoding, offset, length) is ' +
+                 'deprecated. Use write(string[, offset[, length]]' +
+                 '[, encoding]) instead.';
 Buffer.prototype.write = function(string, offset, length, encoding) {
   // Buffer#write(string);
-  if (util.isUndefined(offset)) {
+  if (offset === undefined) {
     encoding = 'utf8';
     length = this.length;
     offset = 0;
 
   // Buffer#write(string, encoding)
-  } else if (util.isUndefined(length) && util.isString(offset)) {
+  } else if (length === undefined && typeof offset === 'string') {
     encoding = offset;
     length = this.length;
     offset = 0;
@@ -384,7 +679,7 @@ Buffer.prototype.write = function(string, offset, length, encoding) {
     offset = offset >>> 0;
     if (isFinite(length)) {
       length = length >>> 0;
-      if (util.isUndefined(encoding))
+      if (encoding === undefined)
         encoding = 'utf8';
     } else {
       encoding = length;
@@ -393,16 +688,7 @@ Buffer.prototype.write = function(string, offset, length, encoding) {
 
   // XXX legacy write(string, encoding, offset, length) - remove in v0.13
   } else {
-    if (!writeWarned) {
-      if (process.throwDeprecation)
-        throw new Error(writeMsg);
-      else if (process.traceDeprecation)
-        console.trace(writeMsg);
-      else
-        console.error(writeMsg);
-      writeWarned = true;
-    }
-
+    writeWarned = internalUtil.printDeprecationMessage(writeMsg, writeWarned);
     var swap = encoding;
     encoding = offset;
     offset = length >>> 0;
@@ -410,50 +696,48 @@ Buffer.prototype.write = function(string, offset, length, encoding) {
   }
 
   var remaining = this.length - offset;
-  if (util.isUndefined(length) || length > remaining)
+  if (length === undefined || length > remaining)
     length = remaining;
 
-  encoding = !!encoding ? (encoding + '').toLowerCase() : 'utf8';
-
   if (string.length > 0 && (length < 0 || offset < 0))
     throw new RangeError('attempt to write outside buffer bounds');
 
-  var ret;
-  switch (encoding) {
-    case 'hex':
-      ret = this.hexWrite(string, offset, length);
-      break;
+  if (!encoding)
+    encoding = 'utf8';
 
-    case 'utf8':
-    case 'utf-8':
-      ret = this.utf8Write(string, offset, length);
-      break;
+  var loweredCase = false;
+  for (;;) {
+    switch (encoding) {
+      case 'hex':
+        return this.hexWrite(string, offset, length);
 
-    case 'ascii':
-      ret = this.asciiWrite(string, offset, length);
-      break;
+      case 'utf8':
+      case 'utf-8':
+        return this.utf8Write(string, offset, length);
 
-    case 'binary':
-      ret = this.binaryWrite(string, offset, length);
-      break;
+      case 'ascii':
+        return this.asciiWrite(string, offset, length);
 
-    case 'base64':
-      // Warning: maxLength not taken into account in base64Write
-      ret = this.base64Write(string, offset, length);
-      break;
+      case 'binary':
+        return this.binaryWrite(string, offset, length);
 
-    case 'ucs2':
-    case 'ucs-2':
-    case 'utf16le':
-    case 'utf-16le':
-      ret = this.ucs2Write(string, offset, length);
-      break;
+      case 'base64':
+        // Warning: maxLength not taken into account in base64Write
+        return this.base64Write(string, offset, length);
 
-    default:
-      throw new TypeError('Unknown encoding: ' + encoding);
+      case 'ucs2':
+      case 'ucs-2':
+      case 'utf16le':
+      case 'utf-16le':
+        return this.ucs2Write(string, offset, length);
+
+      default:
+        if (loweredCase)
+          throw new TypeError('Unknown encoding: ' + encoding);
+        encoding = ('' + encoding).toLowerCase();
+        loweredCase = true;
+    }
   }
-
-  return ret;
 };
 
 
@@ -465,39 +749,10 @@ Buffer.prototype.toJSON = function() {
 };
 
 
-// TODO(trevnorris): currently works like Array.prototype.slice(), which
-// doesn't follow the new standard for throwing on out of range indexes.
-Buffer.prototype.slice = function(start, end) {
-  var len = this.length;
-  start = ~~start;
-  end = util.isUndefined(end) ? len : ~~end;
-
-  if (start < 0) {
-    start += len;
-    if (start < 0)
-      start = 0;
-  } else if (start > len) {
-    start = len;
-  }
-
-  if (end < 0) {
-    end += len;
-    if (end < 0)
-      end = 0;
-  } else if (end > len) {
-    end = len;
-  }
-
-  if (end < start)
-    end = start;
-
-  var buf = new NativeBuffer();
-  sliceOnto(this, buf, start, end);
-  buf.length = end - start;
-  if (buf.length > 0)
-    buf.parent = util.isUndefined(this.parent) ? this : this.parent;
-
-  return buf;
+Buffer.prototype.slice = function slice(start, end) {
+  const buffer = this.subarray(start, end);
+  Object.setPrototypeOf(buffer, Buffer.prototype);
+  return buffer;
 };
 
 
@@ -681,7 +936,7 @@ Buffer.prototype.readFloatLE = function readFloatLE(offset, noAssert) {
   offset = offset >>> 0;
   if (!noAssert)
     checkOffset(offset, 4, this.length);
-  return internal.readFloatLE(this, offset);
+  return binding.readFloatLE(this, offset);
 };
 
 
@@ -689,7 +944,7 @@ Buffer.prototype.readFloatBE = function readFloatBE(offset, noAssert) {
   offset = offset >>> 0;
   if (!noAssert)
     checkOffset(offset, 4, this.length);
-  return internal.readFloatBE(this, offset);
+  return binding.readFloatBE(this, offset);
 };
 
 
@@ -697,7 +952,7 @@ Buffer.prototype.readDoubleLE = function readDoubleLE(offset, noAssert) {
   offset = offset >>> 0;
   if (!noAssert)
     checkOffset(offset, 8, this.length);
-  return internal.readDoubleLE(this, offset);
+  return binding.readDoubleLE(this, offset);
 };
 
 
@@ -705,7 +960,7 @@ Buffer.prototype.readDoubleBE = function readDoubleBE(offset, noAssert) {
   offset = offset >>> 0;
   if (!noAssert)
     checkOffset(offset, 8, this.length);
-  return internal.readDoubleBE(this, offset);
+  return binding.readDoubleBE(this, offset);
 };
 
 
@@ -723,8 +978,10 @@ Buffer.prototype.writeUIntLE = function(value, offset, byteLength, noAssert) {
   value = +value;
   offset = offset >>> 0;
   byteLength = byteLength >>> 0;
-  if (!noAssert)
-    checkInt(this, value, offset, byteLength, Math.pow(2, 8 * byteLength), 0);
+  if (!noAssert) {
+    const maxBytes = Math.pow(2, 8 * byteLength) - 1;
+    checkInt(this, value, offset, byteLength, maxBytes, 0);
+  }
 
   var mul = 1;
   var i = 0;
@@ -740,8 +997,10 @@ Buffer.prototype.writeUIntBE = function(value, offset, byteLength, noAssert) {
   value = +value;
   offset = offset >>> 0;
   byteLength = byteLength >>> 0;
-  if (!noAssert)
-    checkInt(this, value, offset, byteLength, Math.pow(2, 8 * byteLength), 0);
+  if (!noAssert) {
+    const maxBytes = Math.pow(2, 8 * byteLength) - 1;
+    checkInt(this, value, offset, byteLength, maxBytes, 0);
+  }
 
   var i = byteLength - 1;
   var mul = 1;
@@ -825,10 +1084,13 @@ Buffer.prototype.writeIntLE = function(value, offset, byteLength, noAssert) {
 
   var i = 0;
   var mul = 1;
-  var sub = value < 0 ? 1 : 0;
+  var sub = 0;
   this[offset] = value;
-  while (++i < byteLength && (mul *= 0x100))
+  while (++i < byteLength && (mul *= 0x100)) {
+    if (value < 0 && sub === 0 && this[offset + i - 1] !== 0)
+      sub = 1;
     this[offset + i] = ((value / mul) >> 0) - sub;
+  }
 
   return offset + byteLength;
 };
@@ -848,10 +1110,13 @@ Buffer.prototype.writeIntBE = function(value, offset, byteLength, noAssert) {
 
   var i = byteLength - 1;
   var mul = 1;
-  var sub = value < 0 ? 1 : 0;
+  var sub = 0;
   this[offset + i] = value;
-  while (--i >= 0 && (mul *= 0x100))
+  while (--i >= 0 && (mul *= 0x100)) {
+    if (value < 0 && sub === 0 && this[offset + i + 1] !== 0)
+      sub = 1;
     this[offset + i] = ((value / mul) >> 0) - sub;
+  }
 
   return offset + byteLength;
 };
@@ -915,20 +1180,13 @@ Buffer.prototype.writeInt32BE = function(value, offset, noAssert) {
 };
 
 
-function checkFloat(buffer, value, offset, ext) {
-  if (!(buffer instanceof Buffer))
-    throw new TypeError('buffer must be a Buffer instance');
-  if (offset + ext > buffer.length)
-    throw new RangeError('index out of range');
-}
-
-
 Buffer.prototype.writeFloatLE = function writeFloatLE(val, offset, noAssert) {
   val = +val;
   offset = offset >>> 0;
   if (!noAssert)
-    checkFloat(this, val, offset, 4);
-  internal.writeFloatLE(this, val, offset);
+    binding.writeFloatLE(this, val, offset);
+  else
+    binding.writeFloatLE(this, val, offset, true);
   return offset + 4;
 };
 
@@ -937,8 +1195,9 @@ Buffer.prototype.writeFloatBE = function writeFloatBE(val, offset, noAssert) {
   val = +val;
   offset = offset >>> 0;
   if (!noAssert)
-    checkFloat(this, val, offset, 4);
-  internal.writeFloatBE(this, val, offset);
+    binding.writeFloatBE(this, val, offset);
+  else
+    binding.writeFloatBE(this, val, offset, true);
   return offset + 4;
 };
 
@@ -947,8 +1206,9 @@ Buffer.prototype.writeDoubleLE = function writeDoubleLE(val, offset, noAssert) {
   val = +val;
   offset = offset >>> 0;
   if (!noAssert)
-    checkFloat(this, val, offset, 8);
-  internal.writeDoubleLE(this, val, offset);
+    binding.writeDoubleLE(this, val, offset);
+  else
+    binding.writeDoubleLE(this, val, offset, true);
   return offset + 8;
 };
 
@@ -957,7 +1217,8 @@ Buffer.prototype.writeDoubleBE = function writeDoubleBE(val, offset, noAssert) {
   val = +val;
   offset = offset >>> 0;
   if (!noAssert)
-    checkFloat(this, val, offset, 8);
-  internal.writeDoubleBE(this, val, offset);
+    binding.writeDoubleBE(this, val, offset);
+  else
+    binding.writeDoubleBE(this, val, offset, true);
   return offset + 8;
 };
