@@ -21,17 +21,21 @@
 
 'use strict';
 
+const internalModule = require('internal/module');
+const internalUtil = require('internal/util');
 const util = require('util');
 const inherits = util.inherits;
 const Stream = require('stream');
 const vm = require('vm');
 const path = require('path');
 const fs = require('fs');
-const rl = require('readline');
+const Interface = require('readline').Interface;
 const Console = require('console').Console;
+const Module = require('module');
 const domain = require('domain');
 const debug = util.debuglog('repl');
 
+const parentModule = module;
 const replMap = new WeakMap();
 
 try {
@@ -80,6 +84,7 @@ class LineParser {
     this._literal = null;
     this.shouldFail = false;
     this.blockComment = false;
+    this.regExpLiteral = false;
   }
 
   parseLine(line) {
@@ -95,6 +100,11 @@ class LineParser {
       }
 
       if (!this._literal) {
+        if (this.regExpLiteral && current === '/') {
+          this.regExpLiteral = false;
+          previous = null;
+          continue;
+        }
         if (previous === '*' && current === '/') {
           if (this.blockComment) {
             this.blockComment = false;
@@ -111,13 +121,17 @@ class LineParser {
           break;
         }
 
-        if (previous === '/' && current === '*') {
-          this.blockComment = true;
+        if (previous === '/') {
+          if (current === '*') {
+            this.blockComment = true;
+          } else {
+            this.regExpLiteral = true;
+          }
           previous = null;
         }
       }
 
-      if (this.blockComment) continue;
+      if (this.blockComment || this.regExpLiteral) continue;
 
       if (current === this._literal) {
         this._literal = null;
@@ -180,8 +194,6 @@ function REPLServer(prompt,
     prompt = options.prompt;
     dom = options.domain;
     replMode = options.replMode;
-  } else if (typeof prompt !== 'string') {
-    throw new Error('An options Object, or a prompt String are required');
   } else {
     options = {};
   }
@@ -208,7 +220,7 @@ function REPLServer(prompt,
   eval_ = eval_ || defaultEval;
 
   function defaultEval(code, context, file, cb) {
-    var err, result, retry = false;
+    var err, result, retry = false, input = code, wrappedErr;
     // first, create the Script object to check the syntax
     while (true) {
       try {
@@ -216,7 +228,7 @@ function REPLServer(prompt,
             (self.replMode === exports.REPL_MODE_STRICT || retry)) {
           // "void 0" keeps the repl from returning "use strict" as the
           // result value for let/const statements.
-          code = `'use strict'; void 0; ${code}`;
+          code = `'use strict'; void 0;\n${code}`;
         }
         var script = vm.createScript(code, {
           filename: file,
@@ -226,14 +238,23 @@ function REPLServer(prompt,
         debug('parse error %j', code, e);
         if (self.replMode === exports.REPL_MODE_MAGIC &&
             e.message === BLOCK_SCOPED_ERROR &&
-            !retry) {
-          retry = true;
+            !retry || self.wrappedCmd) {
+          if (self.wrappedCmd) {
+            self.wrappedCmd = false;
+            // unwrap and try again
+            code = `${input.substring(1, input.length - 2)}\n`;
+            wrappedErr = e;
+          } else {
+            retry = true;
+          }
           continue;
         }
-        if (isRecoverableError(e, self))
-          err = new Recoverable(e);
+        // preserve original error for wrapped command
+        const error = wrappedErr || e;
+        if (isRecoverableError(error, self))
+          err = new Recoverable(error);
         else
-          err = e;
+          err = error;
       }
       break;
     }
@@ -274,6 +295,11 @@ function REPLServer(prompt,
   self._domain.on('error', function(e) {
     debug('domain error');
     const top = replMap.get(self);
+    internalUtil.decorateErrorStack(e);
+    if (e.stack && self.replMode === exports.REPL_MODE_STRICT) {
+      e.stack = e.stack.replace(/(\s+at\s+repl:)(\d+)/,
+                                (_, pre, line) => pre + (line - 1));
+    }
     top.outputStream.write((e.stack || e) + '\n');
     top.lineParser.reset();
     top.bufferedCommand = '';
@@ -310,7 +336,7 @@ function REPLServer(prompt,
     self.complete(text, callback);
   }
 
-  rl.Interface.call(this, {
+  Interface.call(this, {
     input: self.inputStream,
     output: self.outputStream,
     completer: complete,
@@ -387,7 +413,7 @@ function REPLServer(prompt,
       var rest = matches && matches[2];
       if (self.parseREPLKeyword(keyword, rest) === true) {
         return;
-      } else {
+      } else if (!self.bufferedCommand) {
         self.outputStream.write('Invalid REPL keyword\n');
         skipCatchall = true;
       }
@@ -401,6 +427,7 @@ function REPLServer(prompt,
         // to wrap it in parentheses, so that it will be interpreted as
         // an expression.
         evalCmd = '(' + evalCmd + ')\n';
+        self.wrappedCmd = true;
       } else {
         // otherwise we just append a \n so that it will be either
         // terminated, or continued onto the next expression if it's an
@@ -418,7 +445,8 @@ function REPLServer(prompt,
       debug('finish', e, ret);
       self.memory(cmd);
 
-      if (e && !self.bufferedCommand && cmd.trim().match(/^npm /)) {
+      self.wrappedCmd = false;
+      if (e && !self.bufferedCommand && cmd.trim().startsWith('npm ')) {
         self.outputStream.write('npm should be run outside of the ' +
                                 'node repl, in your normal shell.\n' +
                                 '(Press Control-D to exit.)\n');
@@ -469,7 +497,7 @@ function REPLServer(prompt,
 
   self.displayPrompt();
 }
-inherits(REPLServer, rl.Interface);
+inherits(REPLServer, Interface);
 exports.REPLServer = REPLServer;
 
 exports.REPL_MODE_SLOPPY = Symbol('repl-sloppy');
@@ -495,6 +523,20 @@ exports.start = function(prompt,
   return repl;
 };
 
+REPLServer.prototype.close = function replClose() {
+  if (this.terminal && this._flushing && !this._closingOnFlush) {
+    this._closingOnFlush = true;
+    this.once('flushHistory', () =>
+      Interface.prototype.close.call(this)
+    );
+
+    return;
+  }
+  process.nextTick(() =>
+    Interface.prototype.close.call(this)
+  );
+};
+
 REPLServer.prototype.createContext = function() {
   var context;
   if (this.useGlobal) {
@@ -507,6 +549,10 @@ REPLServer.prototype.createContext = function() {
     context.global.global = context;
   }
 
+  const module = new Module('<repl>');
+  module.paths = Module._resolveLookupPaths('<repl>', parentModule)[1];
+
+  const require = internalModule.makeRequireFunction.call(module);
   context.module = module;
   context.require = require;
 
@@ -545,7 +591,8 @@ REPLServer.prototype.displayPrompt = function(preserveCursor) {
   var prompt = this._initialPrompt;
   if (this.bufferedCommand.length) {
     prompt = '...';
-    var levelInd = new Array(this.lines.level.length).join('..');
+    const len = this.lines.level.length ? this.lines.level.length - 1 : 0;
+    const levelInd = '..'.repeat(len);
     prompt += levelInd + ' ';
   }
 
@@ -631,7 +678,7 @@ REPLServer.prototype.complete = function(line, callback) {
   // list of completion lists, one for each inheritance "level"
   var completionGroups = [];
 
-  var completeOn, match, filter, i, group, c;
+  var completeOn, i, group, c;
 
   // REPL commands (e.g. ".break").
   var match = null;
@@ -646,7 +693,7 @@ REPLServer.prototype.complete = function(line, callback) {
     completionGroupsLoaded();
   } else if (match = line.match(requireRE)) {
     // require('...<Tab>')
-    var exts = Object.keys(require.extensions);
+    const exts = Object.keys(this.context.require.extensions);
     var indexRe = new RegExp('^index(' + exts.map(regexpEscape).join('|') +
                              ')$');
 
@@ -897,7 +944,8 @@ REPLServer.prototype.memory = function memory(cmd) {
   // save the line so I can do magic later
   if (cmd) {
     // TODO should I tab the level?
-    self.lines.push(new Array(self.lines.level.length).join('  ') + cmd);
+    const len = self.lines.level.length ? self.lines.level.length - 1 : 0;
+    self.lines.push('  '.repeat(len) + cmd);
   } else {
     // I don't want to not change the format too much...
     self.lines.push('');
@@ -1059,6 +1107,9 @@ function defineDefaultCommands(repl) {
               self.write(line + '\n');
             }
           });
+        } else {
+          this.outputStream.write('Failed to load:' + file +
+                                  ' is not a valid file\n');
         }
       } catch (e) {
         this.outputStream.write('Failed to load:' + file + '\n');
@@ -1084,7 +1135,7 @@ function regexpEscape(s) {
 REPLServer.prototype.convertToContext = function(cmd) {
   const scopeVar = /^\s*var\s*([_\w\$]+)(.*)$/m;
   const scopeFunc = /^\s*function\s*([_\w\$]+)/;
-  var self = this, matches;
+  var matches;
 
   // Replaces: var foo = "bar";  with: self.context.foo = bar;
   matches = scopeVar.exec(cmd);
@@ -1093,9 +1144,9 @@ REPLServer.prototype.convertToContext = function(cmd) {
   }
 
   // Replaces: function foo() {};  with: foo = function foo() {};
-  matches = scopeFunc.exec(self.bufferedCommand);
+  matches = scopeFunc.exec(this.bufferedCommand);
   if (matches && matches.length === 2) {
-    return matches[1] + ' = ' + self.bufferedCommand;
+    return matches[1] + ' = ' + this.bufferedCommand;
   }
 
   return cmd;
@@ -1112,7 +1163,10 @@ function isRecoverableError(e, self) {
       self._inTemplateLiteral = true;
       return true;
     }
-    return /^(Unexpected end of input|Unexpected token)/.test(message);
+
+    return message.startsWith('Unexpected end of input') ||
+      message.startsWith('Unexpected token') ||
+      message.startsWith('missing ) after argument list');
   }
   return false;
 }
