@@ -1,25 +1,10 @@
-// Copyright Joyent, Inc. and other Node contributors.
-//
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the
-// "Software"), to deal in the Software without restriction, including
-// without limitation the rights to use, copy, modify, merge, publish,
-// distribute, sublicense, and/or sell copies of the Software, and to permit
-// persons to whom the Software is furnished to do so, subject to the
-// following conditions:
-//
-// The above copyright notice and this permission notice shall be included
-// in all copies or substantial portions of the Software.
-//
-// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
-// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
-// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
-// USE OR OTHER DEALINGS IN THE SOFTWARE.
+'use strict';
+
+const Buffer = require('buffer').Buffer;
 
 function assertEncoding(encoding) {
+  // Do not cache `Buffer.isEncoding`, some modules monkey-patch it to support
+  // additional encodings
   if (encoding && !Buffer.isEncoding(encoding)) {
     throw new Error('Unknown encoding: ' + encoding);
   }
@@ -33,7 +18,7 @@ function assertEncoding(encoding) {
 // to reason about this code, so it should be split up in the future.
 // @TODO There should be a utf8-strict encoding that rejects invalid UTF-8 code
 // points as used by CESU-8.
-var StringDecoder = exports.StringDecoder = function(encoding) {
+const StringDecoder = exports.StringDecoder = function(encoding) {
   this.encoding = (encoding || 'utf8').toLowerCase().replace(/[-_]/, '');
   assertEncoding(encoding);
   switch (this.encoding) {
@@ -78,65 +63,84 @@ var StringDecoder = exports.StringDecoder = function(encoding) {
 // replacement character. See https://codereview.chromium.org/121173009/ .
 StringDecoder.prototype.write = function(buffer) {
   var charStr = '';
+  var buflen = buffer.length;
+  var charBuffer = this.charBuffer;
+  var charLength = this.charLength;
+  var charReceived = this.charReceived;
+  var surrogateSize = this.surrogateSize;
+  var encoding = this.encoding;
+  var charCode;
   // if our last write ended with an incomplete multibyte character
-  while (this.charLength) {
+  while (charLength) {
     // determine how many remaining bytes this buffer has to offer for this char
-    var available = (buffer.length >= this.charLength - this.charReceived) ?
-        this.charLength - this.charReceived :
-        buffer.length;
+    var diff = charLength - charReceived;
+    var available = (buflen >= diff) ? diff : buflen;
 
     // add the new bytes to the char buffer
-    buffer.copy(this.charBuffer, this.charReceived, 0, available);
-    this.charReceived += available;
+    buffer.copy(charBuffer, charReceived, 0, available);
+    charReceived += available;
 
-    if (this.charReceived < this.charLength) {
+    if (charReceived < charLength) {
       // still not enough chars in this buffer? wait for more ...
+
+      this.charLength = charLength;
+      this.charReceived = charReceived;
+
       return '';
     }
 
     // remove bytes belonging to the current character from the buffer
-    buffer = buffer.slice(available, buffer.length);
+    buffer = buffer.slice(available, buflen);
+    buflen = buffer.length;
 
     // get the character that was split
-    charStr = this.charBuffer.slice(0, this.charLength).toString(this.encoding);
+    charStr = charBuffer.toString(encoding, 0, charLength);
 
     // CESU-8: lead surrogate (D800-DBFF) is also the incomplete character
-    var charCode = charStr.charCodeAt(charStr.length - 1);
+    charCode = charStr.charCodeAt(charStr.length - 1);
     if (charCode >= 0xD800 && charCode <= 0xDBFF) {
-      this.charLength += this.surrogateSize;
+      charLength += surrogateSize;
       charStr = '';
       continue;
     }
-    this.charReceived = this.charLength = 0;
+    charReceived = charLength = 0;
 
     // if there are no more bytes in this buffer, just emit our char
-    if (buffer.length === 0) {
+    if (buflen === 0) {
+      this.charLength = charLength;
+      this.charReceived = charReceived;
+
       return charStr;
     }
-    break;
   }
 
   // determine and set charLength / charReceived
-  this.detectIncompleteChar(buffer);
+  if (this.detectIncompleteChar(buffer))
+    charLength = this.charLength;
+  charReceived = this.charReceived;
 
-  var end = buffer.length;
-  if (this.charLength) {
+  var end = buflen;
+  if (charLength) {
     // buffer the incomplete character bytes we got
-    buffer.copy(this.charBuffer, 0, buffer.length - this.charReceived, end);
-    end -= this.charReceived;
+    buffer.copy(charBuffer, 0, buflen - charReceived, end);
+    end -= charReceived;
   }
 
-  charStr += buffer.toString(this.encoding, 0, end);
+  this.charLength = charLength;
+  charStr += buffer.toString(encoding, 0, end);
 
-  var end = charStr.length - 1;
-  var charCode = charStr.charCodeAt(end);
+  end = charStr.length - 1;
+  charCode = charStr.charCodeAt(end);
   // CESU-8: lead surrogate (D800-DBFF) is also the incomplete character
   if (charCode >= 0xD800 && charCode <= 0xDBFF) {
-    var size = this.surrogateSize;
-    this.charLength += size;
-    this.charReceived += size;
-    this.charBuffer.copy(this.charBuffer, size, 0, size);
-    buffer.copy(this.charBuffer, 0, 0, size);
+    charLength += surrogateSize;
+    charReceived += surrogateSize;
+    charBuffer.copy(charBuffer, surrogateSize, 0, surrogateSize);
+    buffer.copy(charBuffer, 0, 0, surrogateSize);
+
+    this.charLength = charLength;
+    this.charReceived = charReceived;
+
     return charStr.substring(0, end);
   }
 
@@ -149,35 +153,43 @@ StringDecoder.prototype.write = function(buffer) {
 // length that character, and sets this.charReceived to the number of bytes
 // that are available for this character.
 StringDecoder.prototype.detectIncompleteChar = function(buffer) {
+  var buflen = buffer.length;
   // determine how many bytes we have to check at the end of this buffer
-  var i = (buffer.length >= 3) ? 3 : buffer.length;
+  var i = (buflen >= 3) ? 3 : buflen;
+  var newlen = false;
 
   // Figure out if one of the last i bytes of our buffer announces an
   // incomplete char.
   for (; i > 0; i--) {
-    var c = buffer[buffer.length - i];
+    var c = buffer[buflen - i];
 
     // See http://en.wikipedia.org/wiki/UTF-8#Description
 
     // 110XXXXX
-    if (i == 1 && c >> 5 == 0x06) {
+    if (i === 1 && c >> 5 === 0x06) {
       this.charLength = 2;
+      newlen = true;
       break;
     }
 
     // 1110XXXX
-    if (i <= 2 && c >> 4 == 0x0E) {
+    if (i <= 2 && c >> 4 === 0x0E) {
       this.charLength = 3;
+      newlen = true;
       break;
     }
 
     // 11110XXX
-    if (i <= 3 && c >> 3 == 0x1E) {
+    if (i <= 3 && c >> 3 === 0x1E) {
       this.charLength = 4;
+      newlen = true;
       break;
     }
   }
+
   this.charReceived = i;
+
+  return newlen;
 };
 
 StringDecoder.prototype.end = function(buffer) {
@@ -185,11 +197,12 @@ StringDecoder.prototype.end = function(buffer) {
   if (buffer && buffer.length)
     res = this.write(buffer);
 
-  if (this.charReceived) {
-    var cr = this.charReceived;
+  var charReceived = this.charReceived;
+  if (charReceived) {
+    var cr = charReceived;
     var buf = this.charBuffer;
     var enc = this.encoding;
-    res += buf.slice(0, cr).toString(enc);
+    res += buf.toString(enc, 0, cr);
   }
 
   return res;
@@ -200,11 +213,13 @@ function passThroughWrite(buffer) {
 }
 
 function utf16DetectIncompleteChar(buffer) {
-  this.charReceived = buffer.length % 2;
-  this.charLength = this.charReceived ? 2 : 0;
+  var charReceived = this.charReceived = buffer.length % 2;
+  this.charLength = charReceived ? 2 : 0;
+  return true;
 }
 
 function base64DetectIncompleteChar(buffer) {
-  this.charReceived = buffer.length % 3;
-  this.charLength = this.charReceived ? 3 : 0;
+  var charReceived = this.charReceived = buffer.length % 3;
+  this.charLength = charReceived ? 3 : 0;
+  return true;
 }
