@@ -15,7 +15,7 @@ const prompt_npm = 'npm should be run outside of the ' +
                    'node repl, in your normal shell.\n' +
                    '(Press Control-D to exit.)\n';
 const expect_npm = prompt_npm + prompt_unix;
-var server_tcp, server_unix, client_tcp, client_unix, timer;
+var server_tcp, server_unix, client_tcp, client_unix, replServer;
 
 
 // absolute path to test/fixtures/a.js
@@ -45,12 +45,19 @@ function send_expect(list) {
 function clean_up() {
   client_tcp.end();
   client_unix.end();
-  clearTimeout(timer);
+}
+
+function strict_mode_error_test() {
+  send_expect([
+    { client: client_unix, send: 'ref = 1',
+      expect: /^ReferenceError:\sref\sis\snot\sdefined\n\s+at\srepl:1:5/ },
+  ]);
 }
 
 function error_test() {
   // The other stuff is done so reuse unix socket
   var read_buffer = '';
+  var run_strict_test = true;
   client_unix.removeAllListeners('data');
 
   client_unix.on('data', function(data) {
@@ -72,6 +79,10 @@ function error_test() {
       read_buffer = '';
       if (client_unix.list && client_unix.list.length > 0) {
         send_expect(client_unix.list);
+      } else if (run_strict_test) {
+        replServer.replMode = repl.REPL_MODE_STRICT;
+        run_strict_test = false;
+        strict_mode_error_test();
       } else {
         console.error('End of Error test, running TCP test.');
         tcp_test();
@@ -83,6 +94,10 @@ function error_test() {
       read_buffer = '';
       if (client_unix.list && client_unix.list.length > 0) {
         send_expect(client_unix.list);
+      } else if (run_strict_test) {
+        replServer.replMode = repl.REPL_MODE_STRICT;
+        run_strict_test = false;
+        strict_mode_error_test();
       } else {
         console.error('End of Error test, running TCP test.\n');
         tcp_test();
@@ -307,6 +322,8 @@ function error_test() {
     { client: client_unix, send: 'function x(s) {\nreturn s.replace(/.*/,"");\n}',
       expect: prompt_multiline + prompt_multiline +
             'undefined\n' + prompt_unix },
+    { client: client_unix, send: '{ var x = 4; }',
+      expect: 'undefined\n' + prompt_unix },
   ]);
 }
 
@@ -381,12 +398,13 @@ function unix_test() {
       socket.end();
     });
 
-    repl.start({
+    replServer = repl.start({
       prompt: prompt_unix,
       input: socket,
       output: socket,
       useGlobal: true
-    }).context.message = message;
+    });
+    replServer.context.message = message;
   });
 
   server_unix.on('listening', function() {
@@ -444,7 +462,3 @@ function unix_test() {
 }
 
 unix_test();
-
-timer = setTimeout(function() {
-  assert.fail(null, null, 'Timeout');
-}, 5000);
